#
# Sample script for Ontario FRI data.
# Tom Moore, Nov 3, 1995
#

if {![info exists db] || ![info exists $db]} {
    puts "loading inventory file..."
    set db [sdb open x fri.inv]
}

#
# Load in a relational table that converts from numeric WG code to
# and alpha label.  Copy the wg code into an allocated field,
# then get rid of the lookup table.
#
$db join load wg type.rul
$db allocate Wg CODE
$db calculate {Wg=wg}
$db delete wg

#
# Modify some of the fields in the inventory file
#
puts "define variables..."

#
# Get rid of the label field 'YR_UPD' and create a numeric constant field
#
$db define date [set [set db](YR_UPD)]
$db delete YR_UPD

#
# 'age' is defined as the difference between date and origin.  Whenever
# 'date' is change then the value of 'age' will change also.
#
$db define age {date-YR_ORG}

#
# Create a new expression called 'age_class' that will dynamically
# reclassify 'age' into a label.
#
$db define age_class {age?(==,date,"na"):(<,0,"error"):(<,20,"0-19"):(<,40,"20-39"):(<,60,"40-59"):(<,80,"60-79"):(<,100,"80-99"):(<,120,"100-119"):(<,140,"120-139"):"140+"}

#
# Load up the yield curves
#
puts "load yield curves..."
$db curve load v normal.yld

#
# Load the species-site equivalence table.  This describes the relationship
# between the 'Working Group', which is a stand attribute, and the site index
# for each of the species growing on the site.  Some species may not grow
# as well in association with other working groups as they do in pure
# conditions.  This table describes these associations for "SB SW B PJ PO".
# For example, the table indicates that PO found growing on a SB working group
# site class X stand will track along a PO site class 3 yield curve.
# These equivalences will be used in the volume equations.
#
$db join load equiv site.xref

#
# Convert the species list into stocking values and create Volume expressions. 
# According to the information that I have, "B BW CE OC OH PJ PO PW SB SW"
# are the valid species codes for this database.  Unfortunately, I only
# have yield curves for "B PJ PO and SB".  The following variable will
# keep track of substitute curves to use for the species that don't have 
# curves.
#
set volCode(B) B
set volCode(BW) PO
set volCode(CE) B
set volCode(OC) B
set volCode(OH) PO
set volCode(PJ) PJ
set volCode(PO) PO
set volCode(PW) PJ
set volCode(SB) SB
set volCode(SW) SB

#
# Now allocate the variables, according to the list of species names
# previously defined.  Create the volume expression to lookup according
# to the substitute yield curves.  The volume expression will interpolate
# along the correct yield curve, and modify the value by stand and species
# stocking levels.
#
puts "create new stacking and yield variables..."
foreach spp [lsort [array names volCode]] {
    $db allocate Stk_[set spp] NUMBER
    set equiv $volCode([set spp])
    $db define Volume_[set spp] "interpolate(\"v\" (SITE=[set equiv]_equiv),\"$equiv\",\"Age\", age) * STKG * Stk_$spp"
}

#
# Go through the database, and split the SPC field up
# into species components.
#
puts "parse species composition..."
set max [$db info num_records]
for {set i 0} {$i < $max} {incr i} {
    $db position $i
    set spcomp [set [set db](SPC)]
    set len [string length $spcomp]
    for {set j 0} {($j + 3) < $len} {incr j 4} {
	set cp [string trim [string range $spcomp $j [expr $j + 1]]]
	set st [string trim [string range $spcomp [expr $j + 2] [expr $j + 3]]]
	if {$st == 0} {set st 10}
	set [set db](Stk_[set cp]) [expr $st / 10.0]
    }
}

#
# Create a variable that defines total volume
#
puts "define total volume..."
set volList [lsort [array names volCode]]
set volExp "Volume_[lindex $volList 0]"
foreach spp [lrange $volList 1 end] {
    append volExp " + Volume_$spp"
}
$db define Volume $volExp
$db define TotalVolume "Volume*AREA"

#
# Load in the natural succession 'expert' rule
#
source succession.tcl

puts "Done"